@@ -38,7 +38,7 @@ from flent.resultset import ResultSet
 from flent.loggers import get_logger
 
 try:
-    import numpy
+    import numpy as np
     HAS_NUMPY = True
 except ImportError:
     HAS_NUMPY = False
@@ -444,7 +444,7 @@ class Reducer(object):
         if data is None:
             data = resultset[series['data']]
 
-        if 'norm_by' in series:
+        if series and 'norm_by' in series:
             norm_series = series['norm_by'].format(**series)
             norm_data = resultset[norm_series]
         else:
@@ -498,28 +498,28 @@ class MeanReducer(Reducer):
     numpy_req = True
 
     def _reduce(self, data):
-        return numpy.mean(data)
+        return np.mean(data)
 
 
 class MedianReducer(Reducer):
     numpy_req = True
 
     def _reduce(self, data):
-        return numpy.median(data)
+        return np.median(data)
 
 
 class MinReducer(Reducer):
     numpy_req = True
 
     def _reduce(self, data):
-        return numpy.mean(data)
+        return np.mean(data)
 
 
 class MaxReducer(Reducer):
     numpy_req = True
 
     def _reduce(self, data):
-        return numpy.mean(data)
+        return np.mean(data)
 
 
 class SpanReducer(Reducer):
@@ -534,7 +534,7 @@ class MeanSpanReducer(Reducer):
     def _reduce(self, data):
         min_val = min(data)
         d = [i - min_val for i in data]
-        return numpy.mean(d)
+        return np.mean(d)
 
 
 class MeanZeroReducer(Reducer):
@@ -543,13 +543,16 @@ class MeanZeroReducer(Reducer):
 
     def _reduce(self, data):
         d = [p if p is not None else 0 for p in data]
-        return numpy.mean(d) if d else None
+        return np.mean(d) if d else None
 
 
 class RawReducer(Reducer):
 
-    def _get_series(self, resultset, name):
+    def _get_series(self, resultset, name, ensure='val'):
         data = resultset.raw_values[name]
+        if ensure:
+            data = [d for d in data if ensure in d]
+
         if data and self.cutoff is not None:
             start, end = self.cutoff
             min_t = min((d['t'] for d in data))
@@ -559,6 +562,7 @@ class RawReducer(Reducer):
                 return [d for d in data if d['t'] > start_t and d['t'] < end_t]
             else:
                 return [d for d in data if d['t'] > start_t]
+
         return data
 
     def reduce(self, resultset, series, data=None):
@@ -596,7 +600,7 @@ class RawSeqLossReducer(RawReducer):
         if '::' in key:
             key = key.split("::")[0]
         try:
-            data = self._get_series(resultset, key)
+            data = self._get_series(resultset, key, ensure='seq')
             seqs = [d['seq'] for d in data]
 
             return 1 - len(seqs) / (max(seqs) - min(seqs) + 1)
@@ -627,7 +631,7 @@ class MosReducer(RawReducer):
             last_delay = d['val']
             delay_samples.append(d['val'])
 
-        delay = numpy.mean(delay_samples) + 2 * numpy.mean(jitter_samples)
+        delay = np.mean(delay_samples) + 2 * np.mean(jitter_samples)
         lossrate = loss / len(data)
         mos = mos_score(delay, lossrate)
         return mos
