@@ -92,6 +92,40 @@ def load(filename, absolute=False):
     return ResultSet.load_file(filename, absolute)
 
 
+class SeparatorDict(dict):
+    "Dictionary that supports getting nested keys with a separator"
+
+    def __init__(self, *args, **kwargs):
+        self._sep = None
+        if 'sep' in kwargs:
+            self._sep = kwargs['sep']
+            del kwargs['sep']
+        super(SeparatorDict, self).__init__(*args, **kwargs)
+
+    def __getitem__(self, key):
+        if key in self or self._sep is None or \
+           not hasattr(key, "split") or self._sep not in key:
+            return super(SeparatorDict, self).__getitem__(key)
+
+        # Try to walk the metadata structure by the :-separated keys in 'key'.
+        # This makes it possible to extract arbitrary metadata strings from
+        # the structure.
+        try:
+            parts = key.split(self._sep)
+            data = self[parts[0]]
+            parts = parts[1:]
+            while parts:
+                k = parts.pop(0)
+                try:
+                    i = int(k)
+                    data = data[i]
+                except ValueError:
+                    data = data[k]
+            return data
+        except (KeyError, IndexError, TypeError):
+            raise KeyError
+
+
 class ResultSet(object):
 
     def __init__(self, SUFFIX=SUFFIX, **kwargs):
@@ -101,7 +135,8 @@ class ResultSet(object):
         self._loaded_from = None
         self._absolute = False
         self._raw_values = {}
-        self.metadata = kwargs
+        self._raw_keys = None
+        self.metadata = SeparatorDict(kwargs, sep=":")
         self.SUFFIX = SUFFIX
         self.t0 = None
         if 'TIME' not in self.metadata or self.metadata['TIME'] is None:
@@ -131,25 +166,7 @@ class ResultSet(object):
         if key:
             if value is not _EMPTY:
                 self.metadata[key] = value
-            if key in self.metadata:
-                return self.metadata[key]
-            # Try to walk the metadata structure by the :-separated keys in 'key'.
-            # This makes it possible to extract arbitrary metadata strings from
-            # the structure.
-            try:
-                parts = key.split(":")
-                data = self.metadata[parts[0]]
-                parts = parts[1:]
-                while parts:
-                    k = parts.pop(0)
-                    try:
-                        i = int(k)
-                        data = data[i]
-                    except ValueError:
-                        data = data[k]
-                return data
-            except (KeyError, IndexError, TypeError):
-                raise KeyError
+            return self.metadata[key]
         return self.metadata
 
     def label(self):
@@ -175,7 +192,8 @@ class ResultSet(object):
         self._raw_values[name] = data
 
     def set_raw_values(self, raw_values):
-        self._raw_values = raw_values
+        self._raw_values = {k: [SeparatorDict(x, sep="::") for x in v]
+                            for k, v in raw_values.items()}
 
     def get_raw_values(self):
         return self._raw_values
@@ -237,19 +255,29 @@ class ResultSet(object):
         self.t0 = timegm(self.metadata['T0'].timetuple(
         )) + self.metadata['T0'].microsecond / 1000000.0
 
-    def raw_series(self, name, smooth=None, absolute=False):
+    def num_missing(self, name):
+        if name in self.raw_values:
+            seqnos = [i['seq'] for i in self.raw_values[name] if 'seq' in i]
+            if seqnos:
+                return 1 + max(seqnos) - len(seqnos)
+        return 0
+
+    def raw_series(self, name, absolute=False, raw_key=None):
         if name not in self.raw_values:
-            logger.warning("Missing data points for series '%s'", name)
-            return ([], [])
+            raise KeyError()
+
+        if raw_key is None:
+            raw_key = 'val'
+
         if self.t0 is None:
             self._calculate_t0()
-        x_values, y_values = [], []
+
         for i in self.raw_values[name]:
-            x_values.append(i['t'] if absolute else i['t'] - self.t0)
-            y_values.append(i['val'])
-        if smooth:
-            y_values = self.smoothed(y_values, smooth)
-        return x_values, y_values
+            x = i['t'] if absolute else i['t'] - self.t0
+            try:
+                yield x, i[raw_key]
+            except KeyError:
+                continue
 
     def __getitem__(self, name):
         return self.series(name)
@@ -273,6 +301,29 @@ class ResultSet(object):
     def series_names(self):
         return list(self._results.keys())
 
+    @property
+    def raw_keys(self):
+        if self._raw_keys is not None:
+            return self._raw_keys
+
+        raw_keys = {}
+
+        def extract_keys(d, prefix=''):
+            keys = []
+            for k, v in d.items():
+                kn = prefix + k
+                keys.append(kn)
+                if hasattr(v, 'keys'):
+                    keys.extend(extract_keys(v, kn + '::'))
+            return keys
+
+        for k, v in self.raw_values.items():
+            rk = set()
+            for i in v:
+                rk = rk.union(extract_keys(i))
+            raw_keys[k] = rk
+        return raw_keys
+
     def zipped(self, keys=None):
         if keys is None:
             keys = self.series_names
@@ -465,6 +516,7 @@ class ResultSet(object):
 
     @classmethod
     def load_file(cls, filename, absolute=False):
+        logger.debug("Loading data file %s", filename)
         try:
             if filename.endswith(".gz"):
                 o = gzip_open
@@ -477,8 +529,9 @@ class ResultSet(object):
             r._loaded_from = os.path.realpath(filename)
             fp.close()
             return r
-        except IOError:
-            raise RuntimeError("Unable to read input file: '%s'" % filename)
+        except IOError as e:
+            raise RuntimeError("Unable to read input file '%s': %s"
+                               % (filename, e))
 
     @classmethod
     def loads(cls, s):
